@@ -798,8 +798,8 @@ def print_banner(copyright):
                               '"license" for more information.')
 
 STDLIB_WARNING = """\
-debug: WARNING: Library path not found, using compiled-in sys.path.
-debug: WARNING: 'sys.prefix' will not be set.
+debug: WARNING: Library path not found, using compiled-in sys.path, with
+debug: WARNING: sys.prefix = %r
 debug: WARNING: Make sure the pypy binary is kept inside its tree of files.
 debug: WARNING: It is ok to create a symlink to it from somewhere else."""
 
@@ -818,7 +818,8 @@ def setup_bootstrap_path(executable):
     executable = sys.pypy_find_executable(executable)
     stdlib_path = sys.pypy_find_stdlib(executable)
     if stdlib_path is None:
-        print >> sys.stderr, STDLIB_WARNING
+        print >> sys.stderr, STDLIB_WARNING % (
+            getattr(sys, 'prefix', '<missing>'),)
     else:
         sys.path[:] = stdlib_path
     # from this point on, we are free to use all the unicode stuff we want,
