@@ -38,9 +38,16 @@ class W_Library(W_Root):
         space = self.space
         return space.newtext("<clibrary '%s'>" % self.name)
 
+    def check_closed(self):
+        if self.handle == rffi.cast(DLLHANDLE, 0):
+            raise oefmt(self.space.w_ValueError,
+                        "library '%s' has already been closed",
+                        self.name)
+
     @unwrap_spec(w_ctype=W_CType, name='text')
     def load_function(self, w_ctype, name):
         from pypy.module._cffi_backend import ctypeptr, ctypearray
+        self.check_closed()
         space = self.space
         #
         if not isinstance(w_ctype, ctypeptr.W_CTypePtrOrArray):
@@ -60,6 +67,7 @@ class W_Library(W_Root):
 
     @unwrap_spec(w_ctype=W_CType, name='text')
     def read_variable(self, w_ctype, name):
+        self.check_closed()
         space = self.space
         try:
             cdata = dlsym(self.handle, name)
@@ -71,6 +79,7 @@ class W_Library(W_Root):
 
     @unwrap_spec(w_ctype=W_CType, name='text')
     def write_variable(self, w_ctype, name, w_value):
+        self.check_closed()
         space = self.space
         try:
             cdata = dlsym(self.handle, name)
@@ -80,6 +89,9 @@ class W_Library(W_Root):
                         name, self.name)
         w_ctype.convert_from_object(rffi.cast(rffi.CCHARP, cdata), w_value)
 
+    def close_lib(self):
+        self._finalize_()
+
 
 W_Library.typedef = TypeDef(
     '_cffi_backend.Library',
@@ -87,6 +99,7 @@ W_Library.typedef = TypeDef(
     load_function = interp2app(W_Library.load_function),
     read_variable = interp2app(W_Library.read_variable),
     write_variable = interp2app(W_Library.write_variable),
+    close_lib = interp2app(W_Library.close_lib),
     )
 W_Library.typedef.acceptable_as_base_class = False
 
