@@ -381,7 +381,7 @@ def absolute_import_try(space, modulename, baselevel, w_fromlist):
                     w_name = space.getitem(w_fromlist, space.newint(i))
                     if not space.isinstance_w(w_name, space.w_text):
                         raise oefmt(space.w_TypeError,
-                            "'Item in ``fromlist'' not a string")
+                            "'Item in ``fromlist'' must be str, not %T", w_name)
                     if try_getattr(space, w_mod, w_name) is None:
                         return None
         return w_mod
@@ -430,7 +430,7 @@ def _absolute_import(space, modulename, baselevel, w_fromlist, tentative):
                     w_name = space.getitem(w_fromlist, space.newint(i))
                     if not space.isinstance_w(w_name, space.w_text):
                         raise oefmt(space.w_TypeError,
-                            "'Item in ``fromlist'' not a string")
+                            "'Item in ``fromlist'' must be str, not %T", w_name)
                     if try_getattr(space, w_mod, w_name) is None:
                         load_part(space, w_path, prefix, space.text0_w(w_name),
                                   w_mod, tentative=1)
@@ -617,7 +617,7 @@ def load_c_extension(space, filename, modulename):
     from pypy.module.cpyext.api import load_extension_module
     log_pyverbose(space, 1, "import %s # from %s\n" %
                   (modulename, filename))
-    load_extension_module(space, filename, modulename)
+    return load_extension_module(space, filename, modulename)
     # NB. cpyext.api.load_extension_module() can also delegate to _cffi_backend
 
 @jit.dont_look_inside
@@ -681,8 +681,8 @@ def load_module(space, w_modulename, find_info, reuse=False):
                         pass
                 return w_mod
             elif find_info.modtype == C_EXTENSION and has_so_extension(space):
-                load_c_extension(space, find_info.filename, space.text_w(w_modulename))
-                return check_sys_modules(space, w_modulename)
+                return load_c_extension(space, find_info.filename,
+                                        space.text_w(w_modulename))
         except OperationError:
             w_mods = space.sys.get('modules')
             space.call_method(w_mods, 'pop', w_modulename, space.w_None)
